tags: TLS, CA, x509

# 02.创建 CA 证书和秘钥

<!-- TOC -->

- [02.创建 CA 证书和秘钥](#02创建-ca-证书和秘钥)
    - [安装 cfssl 工具集](#安装-cfssl-工具集)
    - [创建根证书 (CA)](#创建根证书-ca)
        - [创建配置文件](#创建配置文件)
        - [创建证书签名请求文件](#创建证书签名请求文件)
        - [生成 CA 证书和私钥](#生成-ca-证书和私钥)
    - [分发证书文件](#分发证书文件)
    - [参考](#参考)

<!-- /TOC -->

为确保安全，`kubernetes` 系统各组件需要使用 `x509` 证书对通信进行加密和认证。

CA (Certificate Authority) 是自签名的根证书，用来签名后续创建的其它证书。

本文档使用 `CloudFlare` 的 PKI 工具集 [cfssl](https://github.com/cloudflare/cfssl) 创建所有证书。

注意：如果没有特殊指明，本文档的所有操作**均在 m7-autocv-gpu01 节点上执行**，然后远程分发文件和执行命令。

## 安装 cfssl 工具集

``` bash
sudo mkdir -p /opt/k8s/cert && cd /opt/k8s
wget https://pkg.cfssl.org/R1.2/cfssl_linux-amd64
mv cfssl_linux-amd64 /opt/k8s/bin/cfssl

wget https://pkg.cfssl.org/R1.2/cfssljson_linux-amd64
mv cfssljson_linux-amd64 /opt/k8s/bin/cfssljson

wget https://pkg.cfssl.org/R1.2/cfssl-certinfo_linux-amd64
mv cfssl-certinfo_linux-amd64 /opt/k8s/bin/cfssl-certinfo

chmod +x /opt/k8s/bin/*
export PATH=/opt/k8s/bin:$PATH
```

## 创建根证书 (CA)

CA 证书是集群所有节点共享的，**只需要创建一个 CA 证书**，后续创建的所有证书都由它签名。

### 创建配置文件

CA 配置文件用于配置根证书的使用场景 (profile) 和具体参数 (usage，过期时间、服务端认证、客户端认证、加密等)，后续在签名其它证书时需要指定特定场景。

``` bash
cd /opt/k8s/work
cat > ca-config.json <<EOF
{
  "signing": {
    "default": {
      "expiry": "87600h"
    },
    "profiles": {
      "kubernetes": {
        "usages": [
            "signing",
            "key encipherment",
            "server auth",
            "client auth"
        ],
        "expiry": "87600h"
      }
    }
  }
}
EOF
```
+ `signing`：表示该证书可用于签名其它证书，生成的 `ca.pem` 证书中 `CA=TRUE`；
+ `server auth`：表示 client 可以用该该证书对 server 提供的证书进行验证；
+ `client auth`：表示 server 可以用该该证书对 client 提供的证书进行验证；

### 创建证书签名请求文件

``` bash
cd /opt/k8s/work
cat > ca-csr.json <<EOF
{
  "CN": "kubernetes",
  "key": {
    "algo": "rsa",
    "size": 2048
  },
  "names": [
    {
      "C": "CN",
      "ST": "BeiJing",
      "L": "BeiJing",
      "O": "k8s",
      "OU": "4Paradigm"
    }
  ]
}
EOF
```
+ CN：`Common Name`，kube-apiserver 从证书中提取该字段作为请求的**用户名 (User Name)**，浏览器使用该字段验证网站是否合法；
+ O：`Organization`，kube-apiserver 从证书中提取该字段作为请求用户所属的**组 (Group)**；
+ kube-apiserver 将提取的 User、Group 作为 `RBAC` 授权的用户标识；

### 生成 CA 证书和私钥

``` bash
cd /opt/k8s/work
cfssl gencert -initca ca-csr.json | cfssljson -bare ca
ls ca*
```

## 分发证书文件

将生成的 CA 证书、秘钥文件、配置文件拷贝到**所有节点**的 `/etc/kubernetes/cert` 目录下：

``` bash
cd /opt/k8s/work
source /opt/k8s/bin/environment.sh # 导入 NODE_IPS 环境变量
for node_ip in ${NODE_IPS[@]}
  do
    echo ">>> ${node_ip}"
    ssh root@${node_ip} "mkdir -p /etc/kubernetes/cert"
    scp ca*.pem ca-config.json root@${node_ip}:/etc/kubernetes/cert
  done
```

## 参考

1. 各种 CA 证书类型：https://github.com/kubernetes-incubator/apiserver-builder/blob/master/docs/concepts/auth.md